#!/bin/sh

cd "$(dirname "$0")"

echo "Change to $(dirname "$0")"

echo "<<<<====...BUCHLA LEM218 PROGRAMMER....=====>>>"

var=$(pwd)
echo "The current working directory $var."

#check for libusb
echo "---looking for libusb---"

NEEDDL=0

LIB=/usr/local/opt/libusb
BFU=/Applications/BuchlaFirmwareUtility.app
DFUPATH=/Applications/BuchlaFirmwareUtility.app/Contents/MacOS/dfu-programmer

if test -d "$LIB"; then
    echo "$LIB exists."
    DFUPATH=./dfu-programmer
elif test -d "$BFU"; then
    echo "$BFU exists."
else
    echo "no libusb"
    NEEDDL=1
fi

if [[ "$NEEDDL" -eq 1 ]]; then
  curl https://buchla.com/firmwarefiles/BuchlaFirmwareUtility_1.8_build_9.zip --output BuchlaFirmwareUtility.zip
  unzip BuchlaFirmwareUtility.zip
  mv BuchlaFirmwareUtility.app /Applications
else
    echo "Not downloading Buchla Firmware Utility"
fi

echo "Using DFU at $DFUPATH"

echo "---libusb installed, proceed---"


#get the hex files in the directory

files=( *.hex )
count=${#files[@]}
echo "count: $count"

#if there's multiple hex files, ask for a selection
if [[ $count>1 ]]; then

  PS3='Select file to flash to LEM218, or 0 to exit: '
  select file in "${files[@]}"; do
      if [[ $REPLY == "0" ]]; then
          echo 'Exiting, Bye!' >&2
          exit
      elif [[ -z $file ]]; then
          echo 'Invalid choice, try again' >&2
      else
          echo "you selected file #$REPLY: $file."
          break
      fi
  done
#otherwise, we'll use whatever hex is in there:
else
  file=${files[0]}
  echo "There's just one firmware file and it's $file"
fi

echo "Preparing to flash $file..."
echo "====.......====="
echo "Sending Sysex using https://github.com/gbevin/SendMIDI"

#put LEM in dfu mode
./sendmidi dec
./sendmidi dev 218e syx 0 2 55 2 1 1
sleep 2
echo "Press any key to continue."
read keyin

#confirm that the 218 is no longer a MIDI device:
echo "MIDI ports now:"
./sendmidi list
echo "..."
sleep 1

echo "Erasing chip..."

$DFUPATH at32uc3b1256 erase

if [ "$?" != "0" ]; then
    echo "Exiting on error!" 1>&2
    exit 1
fi
sleep 1
echo "Programming firmware for 218e-V3..."
$DFUPATH at32uc3b1256 flash --suppress-bootloader-mem  $file
sleep 3
echo "Press any key to restart the 218e."
read keyin

echo "Restarting..."
$DFUPATH at32uc3b1256 start
echo "---->FINISHED programming LEM218."
